local f = CreateFrame("Frame");
local match = string.match;

--# CONFIG

-- textures
local tNormal = [=[Interface\Buttons\UI-Quickslot2]=]; -- normal border
local tBorder = [=[Interface\AddOns\Buffy\border]=];   -- tench and debuff border overlay

-- scale and scale related
local debuffScale = 1; -- debuff button scale, default is 1

-- duration
local timerSeconds = "%ds"; -- format for seconds
local timerMinutes = "%dm"; -- format for minutes
local timerHours   = "%dh"; -- format for hours
local timerDays    = "%dd"; -- format for days, %d is the value
local customTimers = true;  -- custom look for the buff timers, uses the above formats (true = on; false = off)

--# CONFIG END


-- skin the buff buttons!
local function SkinBuffs(buff, count, scale)
  for i = 1, count do
    local n = buff .. i;
    local button = _G[n];
    
    -- make sure it's not doing this for nothing
    if button and not button.buffy then
      button.buffy = true;
      
      local border = _G[n .. "Border"];
      local icon = _G[n .. "Icon"];
      local normal;
      
      -- scale
      if scale then
        button:SetScale(scale);
      end
      
      -- icon
      icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
      icon:ClearAllPoints();
      icon:SetPoint("TOPLEFT", button, "TOPLEFT", 2, -2);
      icon:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -2, 2);
      
      -- debuff and tench border
      if border then
        border:SetTexture(tBorder);
        border:SetTexCoord(0, 1, 0, 1);
        border:SetBlendMode("ADD");
        border:ClearAllPoints();
        border:SetPoint("TOPLEFT", icon, "TOPLEFT", -11, 12);
        border:SetPoint("BOTTOMRIGHT", icon, "BOTTOMRIGHT", 11, -11);
        
        -- hackish. default tench buttons have their own border texture
        if buff == "TempEnchant" then
          border:SetVertexColor(0.7, 0.1, 0.8);
        end
      end
      
      -- normal border
      normal = button:CreateTexture(n .. "Normal", "BORDER");
      normal:SetTexture(tNormal);
      normal:SetPoint("TOPLEFT", icon, "TOPLEFT", -11, 11);
      normal:SetPoint("BOTTOMRIGHT", icon, "BOTTOMRIGHT", 11, -11);
    end
    
    if button and n == "DebuffButton1" then
      button:SetPoint("TOPRIGHT", ConsolidatedBuffs, "BOTTOMRIGHT", 0, -60 / scale);
      --button.SetPoint = function() return; end;
    end
  end
end

local function Buffy_Event(self, event)
  -- scale and reposition!
  if event == "PLAYER_LOGIN" then
    BuffFrame:SetScale(0.8);
    TemporaryEnchantFrame:SetScale(0.8);
    TemporaryEnchantFrame:ClearAllPoints();
    TemporaryEnchantFrame:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -216, -38);
    ConsolidatedBuffs:SetScale(0.8);
    ConsolidatedBuffs:ClearAllPoints();
    ConsolidatedBuffs:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -216, -38);
    
    -- hook the duration update function
    if customTimers then
      hooksecurefunc("AuraButton_UpdateDuration", function(button, timeLeft)
        if not timeLeft then
          return;
        end
        
        -- copied (from pre 3.3.3 UIParent.lua) and altered
        if timeLeft >= 86400 then
          timeLeft = ceil(timeLeft / 86400);
          button.duration:SetFormattedText(timerDays, timeLeft);
        elseif timeLeft >= 3600 then
          timeLeft = ceil(timeLeft / 3600);
          button.duration:SetFormattedText(timerHours, timeLeft);
        elseif timeLeft >= 60 then
          timeLeft = ceil(timeLeft / 60);
          button.duration:SetFormattedText(timerMinutes, timeLeft);
        else
          button.duration:SetFormattedText(timerSeconds, timeLeft);
        end
      end);
    end
  else
    SkinBuffs("TempEnchant", 2);
    SkinBuffs("BuffButton", BUFF_ACTUAL_DISPLAY);
    SkinBuffs("DebuffButton", DEBUFF_ACTUAL_DISPLAY, debuffScale);
  end
end

f:RegisterEvent("PLAYER_LOGIN");
f:RegisterEvent("PLAYER_ENTERING_WORLD");
f:RegisterEvent("UNIT_AURA");

f:SetScript("OnEvent", Buffy_Event);